@@ -1,3 +1,24 @@
+/*
+   PstoeditQtGUI.cpp : This file is part of pstoedit. Implementation of the QT GUI.
+  
+   Copyright (C) 1993 - 2025 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+*/
+
 #include "PstoeditQtGui.h"
 #include <QSpinBox>
 #include <QCheckBox>
@@ -24,6 +45,14 @@
 
 static constexpr char separatorInFormalList = ';';
 
+static const QString pstoeditWeb("http://www.pstoedit.com/");
+static const QString pstoeditWebS("https://www.pstoedit.com/");
+// Two URLS needed above. The http one is needed for retrievel of the latest_version.txt file
+// because the pstoedit homepage has no https certificate (yet) and the QT get methods fail
+// in that case. So for this the http version is used combined with  
+// QNetworkRequest::NoLessSafeRedirectPolicy - see below in CheckForUpdates()
+static const QString pstoeditGithub("https://github.com/woglu/pstoedit/");
+
 static QString GetValueOfOption(const OptionBase* theOption) {
 	// using streams because returning a std:string from the DLL
 	// causes issues with memory management.
@@ -44,6 +73,7 @@ PstoeditQtGui::PstoeditQtGui(int argc_p, char ** argv_p, std::ostream& logStream
 	QObject::connect(ui.actionSelectFiles, &QAction::triggered, this, &PstoeditQtGui::getInputFiles);
 	QObject::connect(ui.actionOpen_Help_Manual, &QAction::triggered, this, &PstoeditQtGui::OpenGUIHelpDocument);
 	QObject::connect(ui.actionCheck_for_Updates, &QAction::triggered, this, &PstoeditQtGui::CheckForUpdates);
+	QObject::connect(ui.actionGet_Support_or_open_a_Ticket, &QAction::triggered, this, &PstoeditQtGui::Get_Support_or_open_a_Ticket);
 	QObject::connect(ui.actionSupport_Pstoedit_Development_and_Maintenance, &QAction::triggered, this, &PstoeditQtGui::SupportPstoeditDevelopmentandMaintenance);
 	QObject::connect(ui.actionAbout, &QAction::triggered, this, &PstoeditQtGui::About);
 	QObject::connect(ui.actionAbout_Qt, &QAction::triggered, this, &QApplication::aboutQt);
@@ -53,17 +83,17 @@ PstoeditQtGui::PstoeditQtGui(int argc_p, char ** argv_p, std::ostream& logStream
 	if (docdir) {
 		abs_path = std::filesystem::absolute(docdir); // for Linux - docdir is null otherwise
 	} else {
-		char * path_to_exe = cppstrdup(argv[0]);
-		char* lastSlash = nullptr;
-		// replace \\ with / and find last /
-		for (char* cp = path_to_exe; *cp; cp++) {
+		char * const path_to_exe = cppstrdup(argv[0]);
+		if (path_to_exe && strlen(path_to_exe)) {
+		  char* lastSlash = nullptr;
+		  // replace \\ with / and find last /
+		  for (char* cp = path_to_exe; *cp; cp++) {
 			if (*cp == '\\') *cp = '/';
 			if (*cp == '/') lastSlash = cp;
-		}
-		if (lastSlash) {
+		  }
+		  if (lastSlash) {
 			*lastSlash = '\0';
-		}
-		if (strlen(path_to_exe)) {
+		  }
 		  abs_path = std::filesystem::absolute(path_to_exe);
 		} else {
 		  abs_path = std::filesystem::current_path(); 
@@ -78,6 +108,10 @@ PstoeditQtGui::PstoeditQtGui(int argc_p, char ** argv_p, std::ostream& logStream
 #endif
 	UrlOfPstoeditHtml += QString("/pstoedit.htm");
 	UrlOfPstoeditHtml.replace(QString("\\"), QString("/"));
+
+	connect(&HTTPmanager, &QNetworkAccessManager::finished,
+		    this, &PstoeditQtGui::ReplyFinished);
+	HTTPmanager.setAutoDeleteReplies(true);
 }
 
 void PstoeditQtGui::fillTabWithOptions(int new_tab_index, ProgramOptions* options,  int sheet, unsigned int opttype, QFormLayout* layout, const char* anchorprefix)
@@ -234,7 +268,7 @@ void PstoeditQtGui::mapoptionstodialog() {
 	// found.
 
 	// box for format selection
-	const DriverDescription_S* ptr = getPstoeditDriverInfo_plainC();
+	DriverDescription_S* ptr = getPstoeditDriverInfo_plainC();
 	const DriverDescription_S* driverInfoIterator = ptr;
 	formatSelector = new QComboBox(ui.tab_ifo);
 	int currentGroup = 1;
@@ -308,6 +342,7 @@ void PstoeditQtGui::mapoptionstodialog() {
 	ui.tabWidget->setCurrentWidget(ui.tab_ifo);
 
 	updateOutputSuffix();
+	clearPstoeditDriverInfo_plainC(ptr);
 }
 
 void PstoeditQtGui::quit() {
@@ -332,8 +367,9 @@ void PstoeditQtGui::updateOutputSuffix() {
 	char* colon1 = std::strchr(tmpbuf, separatorInFormalList);
 	*colon1 = '\0';// copy part up to first '|'
 	const char* driverName = tmpbuf; // from start to :
-	driverOptions = getProgramOptionsForDriver(driverName);
-	// FIXME - delete former options
+	deleteProgramOptionsForDriver(driverOptions); // delete old options if any
+ 	driverOptions = getProgramOptionsForDriver(driverName);
+	
 	
 	if (driverOptions && driverOptions->numberOfOptions()) {
 		//logStream << "The driver for this output format supports the following additional options: (specify using -f \"format:-option1 -option2\")" << endl;
@@ -503,7 +539,7 @@ void PstoeditQtGui::setProgramOptions(ProgramOptions* param) {
 	verbose = (verboseOption && (GetValueOfOption(verboseOption) == "1"));
 }
 
-void PstoeditQtGui::OpenGUIHelpDocument() {
+void PstoeditQtGui::OpenGUIHelpDocument() const {
 	QString helpfileURL = UrlOfPstoeditHtml;
 	helpfileURL += QString("#GUI");
 	QDesktopServices::openUrl(QUrl(helpfileURL, QUrl::TolerantMode));
@@ -512,15 +548,37 @@ void PstoeditQtGui::OpenGUIHelpDocument() {
 
 void PstoeditQtGui::ReplyFinished(QNetworkReply* reply) {
 	const QString answer = reply->readAll();
+	if (!answer.length() || answer.startsWith("404")) {
+		if (Verbose()) {
+			cout << "Could not retrieve latest_version.txt from server." << endl;
+		}
+		QMessageBox::information(this, QString("Pstoedit"), QString("Could not retrieve latest_version.txt from server."));
+		return;
+	}
 	const double latest_version = answer.toFloat();
-	const double current_version = QString(get_pstoedit_version()).toFloat();
+	const char * const my_version_string = get_pstoedit_version();
+	if (!my_version_string || !strlen(my_version_string)) {
+		if (Verbose()) {
+			cout << "Could not retrieve current pstoedit version." << endl;
+		}
+		QMessageBox::information(this, QString("Pstoedit"), QString("Could not retrieve current pstoedit version."));
+		return;
+	}
+	const double current_version = QString(my_version_string).toFloat();
 	if (Verbose()) {
-		cout << "version.txt has: " << answer.toStdString().c_str() << ":" << latest_version << " your version is " << current_version << endl;
+		cout << "latest_version.txt has: " << answer.toStdString().c_str() << ":" << latest_version << " your version is " << current_version << endl;
 	}
 	if (latest_version > current_version) {
 		std::stringstream message;
-		message << "New version " << answer.toStdString().c_str() << " is available. Your version is " << current_version << endl;
-		message << "Get new version from https://sourceforge.net/projects/pstoedit/files/pstoedit/" << endl;
+		message << "New version " << answer.toStdString().c_str() << " is available. Your version is " << my_version_string << endl;
+		message << "Get new version from  " << pstoeditGithub.toStdString() << "releases" << endl; 
+#if defined(_WIN32) || defined(_GLIBCXX_FILESYSTEM_IS_WINDOWS) 
+		message << "or run ´winget install pstoedit' from a terminal" << endl; 
+#endif
+		QMessageBox::information(this, QString("Pstoedit"), message.str().c_str());
+	} else if (latest_version < current_version) {
+		std::stringstream message;
+		message << "Your version " << current_version << " is newer than the released version " << answer.toStdString().c_str() << endl;
 		QMessageBox::information(this, QString("Pstoedit"), message.str().c_str());
 	} else {
 		QMessageBox::information(this, QString("Pstoedit"), QString("Your version is up to date."));
@@ -528,12 +586,9 @@ void PstoeditQtGui::ReplyFinished(QNetworkReply* reply) {
 }
 
 void PstoeditQtGui::CheckForUpdates() {
-	auto status = connect(&HTTPmanager, &QNetworkAccessManager::finished,
-						  this, &PstoeditQtGui::ReplyFinished);
-	if (Verbose()) {
-		cout << "Connection status:" << status << endl;
-	}
-	HTTPmanager.get(QNetworkRequest(QUrl("https://wglunz.users.sourceforge.net/version.txt")));
+	QNetworkRequest request(QUrl(pstoeditWeb + "latest_version.txt"));
+	request.setMaximumRedirectsAllowed(QNetworkRequest::NoLessSafeRedirectPolicy);
+	HTTPmanager.get(request);
 };
 
 void PstoeditQtGui::About() {
@@ -541,9 +596,9 @@ void PstoeditQtGui::About() {
 	message << "**About pstoedit**\n\n"
 		       "Pstoedit version: ";
 	message << get_pstoedit_version() << "  \n" ;
-	message <<	"Copyright (C) 1993 - 2023 Wolfgang Glunz " << "  \n"
+	message <<	"Copyright (C) 1993 - 2025 Wolfgang Glunz " << "  \n"
 				"All rights reserved" << "  \n"
-				"Pstoedit home page:  [www.pstoedit.com](http://www.pstoedit.com \"pstoedit home page\")" << "  \n"
+				"Pstoedit home page: " << QUrl(pstoeditWebS).toDisplayString().toStdString()  << "  \n"
 				"Refer to license.txt for conditions of distribution and use.\n\n"
 				"This binary uses Qt version " << qVersion() << "  \n"
 				"See Help->About Qt for more information about Qt." << endl;
@@ -555,7 +610,11 @@ void PstoeditQtGui::About() {
 	aboutBox.exec();
 };
 
-void PstoeditQtGui::SupportPstoeditDevelopmentandMaintenance() {
-	QDesktopServices::openUrl(QUrl("http://www.pstoedit.com/pstoedit_donate.htm", QUrl::TolerantMode));
+void PstoeditQtGui::SupportPstoeditDevelopmentandMaintenance() const {
+	QDesktopServices::openUrl(QUrl(pstoeditWebS + "pstoedit_donate.htm", QUrl::StrictMode));
+};
+
+void PstoeditQtGui::Get_Support_or_open_a_Ticket() const {
+	QDesktopServices::openUrl(QUrl(pstoeditGithub + "issues", QUrl::StrictMode));
 };
 
