@@ -1,7 +1,7 @@
 /*
    PstoeditQtGUI.h : This file is part of pstoedit. Implementation of the QT GUI.
   
-   Copyright (C) 1993 - 2024 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+   Copyright (C) 1993 - 2025 Wolfgang Glunz, wglunz35_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -85,7 +85,7 @@ class PstoeditQtGui : public QMainWindow
     enum class ButtonState { FullyDisabled, StartEnabled, StopEnabled };
 public:
     PstoeditQtGui(int argc_p, char ** argv_p, std::ostream& logStream_p, std::ostream* debugStream_p, QWidget* parent = Q_NULLPTR);
-    ~PstoeditQtGui() { }
+    ~PstoeditQtGui() { deleteProgramOptionsForDriver(driverOptions); }
     bool Verbose() const { return verbose; }
     bool checkGhostscript();
     void setProgramOptions(ProgramOptions* param);
