@@ -3,7 +3,7 @@
    Backend for MetaPost files
    Contributed by: Scott Pakin <scott+ps2ed_AT_pakin.org>
 
-   Copyright (C) 1993 - 2023 Wolfgang Glunz, wglunz35_AT_geocities.com
+   Copyright (C) 1993 - 2024 Wolfgang Glunz, wglunz35_AT_geocities.com
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ constructBase,
 	outf << "% Converted from PostScript(TM) to MetaPost by pstoedit\n"
 		<<
 		"% MetaPost backend contributed by Scott Pakin <scott+ps2ed_AT_pakin.org>\n"
-		<< "% pstoedit is Copyright (C) 1993 - 2023 Wolfgang Glunz" <<
+		<< "% pstoedit is Copyright (C) 1993 - 2024 Wolfgang Glunz" <<
 		" <wglunz35_AT_pstoedit.net>\n\n";
 
 	/*
@@ -259,7 +259,7 @@ void drvMPOST::show_text(const TextInfo & textinfo)
 
 	// Change fonts
 	std::string thisFontName(textinfo.currentFontName.c_str());
-	if (thisFontName ==  emptystring ) {	// If we're this brain-damaged, we must be a TeX font
+	if (thisFontName == emptystring ) {	// If we're this brain-damaged, we must be a TeX font
 		thisFontName = textinfo.currentFontFullName.c_str();
 		if (!texshortchar) {
 			outf << "shortchar := char(24);" << endl;	// Cedilla in TeX land
@@ -274,7 +274,7 @@ void drvMPOST::show_text(const TextInfo & textinfo)
 	}
 	if (thisFontName != prevFontName) {
 		outf << "defaultfont := \"" << thisFontName << "\";" << endl;
-		prevFontName = thisFontName;
+		prevFontName = thisFontName.c_str();
 	}
 	// Change colors
 	if (textinfo.currentR != prevR || textinfo.currentG != prevG || textinfo.currentB != prevB) {
