@@ -1,5 +1,26 @@
+/* 
+   drvacad.h : This file is part of pstoedit
+   Some header and class definitions for handing the AutoCAD format
+
+   Copyright (C) 1993 - 2025 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+*/
 #include <cmath>
-static const char  dxf14acadheader_prelayer1 [] =
+static const char dxf14acadheader_prelayer1 [] =
 "  0\n"
 "SECTION\n"
 "  2\n"
@@ -285,10 +306,13 @@ public:
 		DXF_LineType::handle++;
 		return out;
 	}
+	static void init() {
+		scalefactor = 1.0;
+		handle = 0xF0;
+	}
 	static double scalefactor;
 	static int handle;
 private:
-	
 	const char * const m_name;
 	const char * const m_description;
 	const std::vector<double> m_pattern;
@@ -1319,7 +1343,7 @@ static const char  dxf14acadheader_postlayer [] =
 "  2\n"
 "ENTITIES\n";
 
-static const char  dxf14acadspline [] =
+static const char dxf14acadspline [] =
 "  0\n"
 "SPLINE\n"
 "  5\n"
