@@ -84,7 +84,6 @@ int pstoedit_plainC(int argc,
                     const char * const psinterpreter  /* if 0, then pstoedit will look for one using whichpi() */
                    );
 
-/* extern "C" DLLEXPORT const class DriverDescription* const * getPstoeditDriverInfo(ostream& errstream); */
 #ifdef __cplusplus
 extern "C" DLLEXPORT 
 #endif
@@ -118,6 +117,11 @@ extern "C" DLLEXPORT
 class ProgramOptions * getProgramOptionsForDriver(const char* driverName);
 #endif
 
+#ifdef __cplusplus
+extern "C" DLLEXPORT
+void deleteProgramOptionsForDriver(class ProgramOptions* options);
+#endif
+
 #ifdef __cplusplus
 extern "C" DLLEXPORT 
 #endif
